
// [...]


// greylink dc++
const string HashManager::StreamStore::streamName(".gltth");
void HashManager::StreamStore::setCheckSum(TTHStreamHeader& h) {
        h.magic = MAGIC;
        uint32_t sum = 0;
        for (size_t i = 0; i < sizeof(TTHStreamHeader) / sizeof(uint32_t); i++)
                sum ^= ((uint32_t*)&h)[i];
        h.checksum ^= sum;
}

bool HashManager::StreamStore::validateCheckSum(const TTHStreamHeader& h) {
        if (h.magic != MAGIC)
                return false;
        uint32_t sum = 0;
        for (size_t i = 0; i < sizeof(TTHStreamHeader) / sizeof(uint32_t); i++)
                sum ^= ((uint32_t*)&h)[i];
        return (sum == 0);
}
bool HashManager::StreamStore::loadTree(TigerTree& tt, int64_t aFileSize /* = -1 */ ) {
        try {
                int64_t fileSize = (aFileSize == -1)? File::getSize(fileName) : aFileSize;
                if (fileSize < minStreamedFileSize) // that's why minStreamedFileSize never be changed!
                        return false;
                uint64_t timeStamp = File::getTimeStamp(fileName);
                {
                        File stream(fileName + ":" + streamName, File::READ, File::OPEN);
                        size_t sz = sizeof(TTHStreamHeader);
                        TTHStreamHeader h;
                        if (stream.read(&h, sz) != sizeof(TTHStreamHeader))
                                return false;
                        if (!validateCheckSum(h) || (uint64_t)fileSize != h.fileSize || timeStamp != h.timeStamp)
                                return false;
                        size_t datalen = TigerTree::calcBlocks(fileSize, h.blockSize) * TTHValue::BYTES;
                        sz = datalen;
                        AutoArray<uint8_t> buf(datalen);
                        if (stream.read((uint8_t*)buf, sz) != datalen)
                                return false;
                        tt = TigerTree(fileSize, h.blockSize, buf);
                        if(!(tt.getRoot() == h.root))
                                return false;
                }
        } catch (const Exception&) {
                return false;
        }
        return true;
}

bool HashManager::StreamStore::saveTree(const TigerTree& tt) {
        if (!gSETTING(TTH_IN_NTFS_STREAM))
                return false;
        try {
                TTHStreamHeader h;
                h.fileSize = File::getSize(fileName);
                if (h.fileSize < minStreamedFileSize || h.fileSize != (uint64_t)tt.getFileSize())
                        return false; // that's why minStreamedFileSize never be changed!
                h.timeStamp = File::getTimeStamp(fileName);
                h.root = tt.getRoot();
                h.blockSize = tt.getBlockSize();
                setCheckSum(h);
                {
                        File stream(fileName + ":" + streamName, File::WRITE, File::CREATE | File::TRUNCATE);
                        stream.write(&h, sizeof(TTHStreamHeader));
                        stream.write(tt.getLeaves()[0].data, tt.getLeaves().size() * TTHValue::BYTES);
                }
                File::setTimeStamp(fileName, h.timeStamp);
        } catch (const Exception&) {
                return false;
        }
        return true;
}

// [...]
